# -*- coding: utf-8 -*-
"""
    __init__

    Test Suite for Tornado-Babel

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import sys
from unittest import TestSuite, TestLoader
sys.path.append('../')

from test_locale import TestLocale
from test_extract import TestExtract

def test_all():
    loader = TestLoader()
    suite = TestSuite()
    suite.addTests(loader.loadTestsFromTestCase(TestLocale))
    suite.addTests(loader.loadTestsFromTestCase(TestExtract))
    return suite



# -*- coding: utf-8 -*-
"""
    extract

    Extract messages from tornado templates

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import ast

from tornado import escape
from tornado.template import (_UNSET, _DEFAULT_AUTOESCAPE, _TemplateReader,
    _parse, _File, _Expression)


GETTEXT_FUNCTIONS = ('_', '_N', 'gettext', 'ngettext')


class DummyTemplate(object):
    """A template object just used to parse the template string.
    The class resembles tornado.Template but stops at parsing the file.
    """
    def __init__(self, template_string, name="<string>", loader=None,
                 compress_whitespace=None, autoescape=_UNSET):
        self.name = name
        self.compress_whitespace = True
        if autoescape is not _UNSET:
            self.autoescape = autoescape
        elif loader:
            self.autoescape = loader.autoescape
        else:
            self.autoescape = _DEFAULT_AUTOESCAPE
        self.namespace = loader.namespace if loader else {}
        reader = _TemplateReader(name, escape.native_str(template_string))
        self.file = _File(self, _parse(reader, self))


def walk(node):
    """
    Given a template node, walk over all its descendants

    >>> t = DummyTemplate("{{ _('Hello') }}")
    >>> len(list(walk(t.file)))
    3
    >>> t = DummyTemplate("{% if a %}{{ _('Hello') }}{% end %}")
    >>> len(list(walk(t.file)))
    5
    """
    for child in node.each_child():
        if child.each_child():
            for grandchild in walk(child):
                yield grandchild
        yield child


def extract_from_node(expression, gettext_functions=None):
    """Extract localizable strings from the given Template Expression

    :param expression: A node of type tornado.template._Expression
    :param gettext_functions: A list of gettext function names that should be
                              parsed from template
    :return: iterator returning tuple in the format babel wants
    """
    if gettext_functions is None:
        gettext_functions = GETTEXT_FUNCTIONS

    for node in ast.walk(ast.parse(expression.expression)):
        # Recursively walk through all descendant nodes 
        if isinstance(node, ast.Call):
            # If the type is a function call
            if not (
                    isinstance(node.func, ast.Name) and \
                    node.func.id in gettext_functions):
                continue

            strings = []
            for arg in node.args:
                if isinstance(arg, ast.Str):
                    strings.append(arg.s)
                else:
                    strings.append(None)

            for arg in node.keywords:
                strings.append(None)
            if node.starargs is not None:
                strings.append(None)
            if node.kwargs is not None:
                strings.append(None)

            if len(strings) == 1:
                strings, = strings
            else:
                strings = tuple(strings)

            yield expression.line, node.func.id, strings


def extract_tornado(fileobj, keywords, comment_tags, options):
    """Extract messages from Python source code.


    :param fileobj: the seekable, file-like object the messages should be
	            extracted from
    :param keywords: a list of keywords (i.e. function names) that should be
	            recognized as translation functions
    :param comment_tags: a list of translator tags to search for and include
	            in the results. (Not implemented yet)
    :param options: a dictionary of additional options (optional)
    :return: an iterator over ``(lineno, funcname, message, comments)`` tuples
    :rtype: ``iterator``
    """
    template = DummyTemplate(
        fileobj.read(),
        file.name or options.get('name', '<string>'),
    )

    for node in walk(template.file):
        if isinstance(node, _Expression):
            for lineno, func, message in extract_from_node(node):
                # TODO: Implement the comment feature, right now an empty 
                # iterable is returned
                yield lineno, func, message, []



# -*- coding: utf-8 -*-
"""
    mixin

    A mixin class that could be used to use tornadobabel instaed of the
    default tornado requesthandler locale.

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from speaklater import is_lazy_string, make_lazy_string
from tornadobabel import locale


def make_lazy_gettext(lookup_func):
    """Creates a lazy gettext function dispatches to a gettext
    function as returned by `lookup_func`.

    :copyright: (c) 2010 by Armin Ronacher.

    Example:

    >>> translations = {u'Yes': u'Ja'}
    >>> lazy_gettext = make_lazy_gettext(lambda: translations.get)
    >>> x = lazy_gettext(u'Yes')
    >>> x
    lu'Ja'
    >>> translations[u'Yes'] = u'Si'
    >>> x
    lu'Si'
    """
    def lazy_gettext(string, *args, **kwargs):
        if is_lazy_string(string):
            return string
        return make_lazy_string(lookup_func(), string, *args, **kwargs)
    return lazy_gettext


class TornadoBabelMixin(object):
    """A Mixin class that could be used with the request handler to override
    the default tornado.locales with tornadobabel.locale, when the browser
    locale is used.

    If your application handles locales, then you should implement the
    `get_user_locale` method.
    """

    @property
    def _(self):
        """
        A helper to easily get lazy version ugettext
        """
        return make_lazy_gettext(lambda: self.locale.translate)

    def get_browser_locale(self, default="en_US"):
        """Determines the user's locale from Accept-Language header.

        See http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4
        """
        if "Accept-Language" in self.request.headers:
            languages = self.request.headers["Accept-Language"].split(",")
            locales = []
            for language in languages:
                parts = language.strip().split(";")
                if len(parts) > 1 and parts[1].startswith("q="):
                    try:
                        score = float(parts[1][2:])
                    except (ValueError, TypeError):
                        score = 0.0
                else:
                    score = 1.0
                locales.append((parts[0], score))
            if locales:
                locales.sort(key=lambda (l, s): s, reverse=True)
                codes = [l[0] for l in locales]
                return locale.get(*codes)
        return locale.get(default)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
"""
    test_locale

    Test the method in the locale usage

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import os
import unittest

from tornado import web
from tornado import testing
from tornadobabel import locale
from tornadobabel.mixin import TornadoBabelMixin


class IndexHandler(web.RequestHandler):
    def get(self):
        self.render('index.html')


class Indexi18nHandler(TornadoBabelMixin, web.RequestHandler):
    def get_user_locale(self):
        """
        This locale handler is a bit old fashioned and picks up locale from
        the argument in the URL
        """
        _locale = locale.get(self.get_argument('locale', 'en_US'))
        return _locale

    def get(self):
        self.render('index.html')


class TestLocale(testing.AsyncHTTPTestCase):
    def get_app(self):
        test_dir = os.path.abspath(
            os.path.dirname(__file__)
        )
        locale.load_gettext_translations(
            os.path.join(test_dir, 'locales'), 'messages'
        )
        return web.Application([
            ('/', IndexHandler),
            ('/i18n', Indexi18nHandler),
        ])

    def test_0010_locales(self):
        self.assertTrue("es_ES" in locale._supported_locales)
        self.assertTrue("fr_FR" in locale._supported_locales)

        es_ES = locale.get('es_ES')
        self.assertEqual(es_ES.translate('Welcome'), "bienvenido")

    def test_without_i18n(self):
        response = self.fetch('/')
        self.assertEqual(response.code, 200)
        self.assertEqual(response.body, "Welcome\n")

    def test_with_i18n(self):
        response = self.fetch('/i18n')
        self.assertEqual(response.code, 200)
        self.assertEqual(response.body, "Welcome\n")

        response = self.fetch('/i18n?locale=fr_FR')
        self.assertEqual(response.code, 200)
        self.assertEqual(response.body, "bienvenu\n")

        response = self.fetch('/i18n?locale=es_ES')
        self.assertEqual(response.code, 200)
        self.assertEqual(response.body, "bienvenido\n")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
    test_extract

    test Message extraction from template files

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import sys
sys.path.append('../')

from StringIO import StringIO
import unittest

from tornadobabel.extract import extract_tornado


class TestExtract(unittest.TestCase):
    def setUp(self):
        pass

    def test_extract(self):
        template = StringIO("""
        {{ _("Test String") }}
        {{ something }}
        {% block something %}{% end %}
        """)
        strings = list(extract_tornado(template, None, None, {}))
        self.assertEqual(len(strings), 1)

        # Ensure that the string is correct
        string, = strings
        self.assertEqual(string[0], 2)  # Line number
        self.assertEqual(string[1], '_')  # Function name
        self.assertEqual(string[2], "Test String")  # Translatable string

    def test_n_extract(self):
        template = StringIO("""
        {{ _("Test String") }}
        {{ _N("%(num)d apple", "%(num)d apples", count) }}
        {{ something }}
        {% block something %}{% end %}
        """)
        strings = list(extract_tornado(template, None, None, {}))
        self.assertEqual(len(strings), 2)

        # Ensure that the string is correct
        string = strings[1]
        self.assertEqual(string[0], 3)  # Line number
        self.assertEqual(string[1], '_N')  # Function name
        self.assertEqual(string[2], ('%(num)d apple', '%(num)d apples', None))

    def test_nested_extract(self):
        template = StringIO("""
        {% extends 'somethingelse.html' %}
        {% block abc %}
        {{ _("Test String") }}
        {% end %}
        """)

        strings = list(extract_tornado(template, None, None, {}))
        self.assertEqual(len(strings), 1)

        # Ensure that the string is correct
        string, = strings
        self.assertEqual(string[0], 4)  # Line number
        self.assertEqual(string[1], '_')  # Function name
        self.assertEqual(string[2], "Test String")  # Translatable string


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
    Internationalisation using Babel
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    The locale support of tornado as such is pretty basic and does not offer
    support for merging translation catalogs and several other features most 
    multi language applications require.

    This module tries to retain the same API as that of tornado.locale while
    implement the required features with the support of babel.

    .. note::
        CSV Translations are not supported

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :copyright: (c) 2009 by Facebook (Tornado Project)
    :license: BSD, see LICENSE for more details.
"""
import gettext
import logging
import os

from babel.support import Translations
from babel.core import Locale as BabelCoreLocale
from babel import dates, numbers

_default_locale = "en_US"
_translations = {}
_supported_locales = frozenset([_default_locale])
_use_gettext = False


def get(*locale_codes):
    """Returns the closest match for the given locale codes.

    We iterate over all given locale codes in order. If we have a tight
    or a loose match for the code (e.g., "en" for "en_US"), we return
    the locale. Otherwise we move to the next code in the list.

    By default we return en_US if no translations are found for any of
    the specified locales. You can change the default locale with
    set_default_locale() below.
    """
    return Locale.get_closest(*locale_codes)


def set_default_locale(code):
    """Sets the default locale, used in get_closest_locale().

    The default locale is assumed to be the language used for all strings
    in the system. The translations loaded from disk are mappings from
    the default locale to the destination locale. Consequently, you don't
    need to create a translation file for the default locale.
    """
    global _default_locale
    global _supported_locales
    _default_locale = code
    _supported_locales = frozenset(_translations.keys() + [_default_locale])


def load_gettext_translations(directory, domain):
    """Loads translations from gettext's locale tree"""
    global _translations
    global _supported_locales
    global _use_gettext
    _translations = {}
    for lang in os.listdir(directory):
        if lang.startswith('.'):
            continue  # skip .svn, etc
        if os.path.isfile(os.path.join(directory, lang)):
            continue
        try:
            # Load existing translation or Null Translations
            translation = _translations.get(lang, Translations.load())
            if isinstance(translation, gettext.NullTranslations):
                _translations[lang] = Translations.load(
                        directory, [lang], domain
                )
            else:
                _translations[lang].merge(
                        Translations.load(directory, [lang], domain)
                )
        except Exception, e:
            logging.error("Cannot load translation for '%s': %s", lang, str(e))
            continue
    _supported_locales = frozenset(_translations.keys() + [_default_locale])
    _use_gettext = True
    logging.info("Supported locales: %s", sorted(_supported_locales))


class Locale(BabelCoreLocale):
    """Object representing a locale.

    After calling one of `load_translations` or `load_gettext_translations`,
    call `get` or `get_closest` to get a Locale object.
    """
    @classmethod
    def get_closest(cls, *locale_codes):
        """Returns the closest match for the given locale code."""
        for code in locale_codes:
            if not code:
                continue
            code = code.replace("-", "_")
            parts = code.split("_")
            if len(parts) > 2:
                continue
            elif len(parts) == 2:
                code = parts[0].lower() + "_" + parts[1].upper()
            if code in _supported_locales:
                return cls.get(code)
            if parts[0].lower() in _supported_locales:
                return cls.get(parts[0].lower())
        return cls.get(_default_locale)

    @classmethod
    def get(cls, code):
        """Returns the Locale for the given locale code.

        If it is not supported, we raise an exception.
        """
        if not hasattr(cls, "_cache"):
            cls._cache = {}
        if code not in cls._cache:
            assert code in _supported_locales
            translations = _translations.get(code, gettext.NullTranslations())
            locale = cls.parse(code)
            locale.translations = translations
            cls._cache[code] = locale
        return cls._cache[code]

    def translate(self, message, plural_message=None, count=None):
        if plural_message is not None:
            assert count is not None
            return self.translations.ungettext(message, plural_message, count)
        else:
            return self.translations.ugettext(message)

    def format_datetime(self, datetime=None, format='medium', tzinfo=None):
        """
        Return a date formatted according to the given pattern.

        :param datetime: the datetime object; if None, the current date and
                         time is used
        :param format: one of "full", "long", "medium", or "short", or a
                       custom date/time pattern
        :param tzinfo: the timezone to apply to the time for display

        >>> from datetime import datetime
        >>> locale = Locale.parse('pt_BR')
        >>> locale
        <Locale "pt_BR">
        >>> dt = datetime(2007, 04, 01, 15, 30)
        >>> locale.format_datetime(dt)
        u'01/04/2007 15:30:00'
        """
        return dates.format_datetime(datetime, format, tzinfo, self)

    def format_date(self, date=None, format='medium'):
        """
        Return a date formatted according to the locale.

        :param date: the date or datetime object; if None, the current date
                     is used
        :param format: one of "full", "long", "medium", or "short", or a
                       custom date/time pattern
        """
        return dates.format_date(date, format, self)

    def format_time(self, time=None, format='medium', tzinfo=None):
        """
        Return a time formatted according to the locale.

        :param time: the time or datetime object; if None, the current time
                     in UTC is used
        :param format: one of "full", "long", "medium", or "short", or a
                       custom date/time pattern
        :param tzinfo: the time-zone to apply to the time for display
        """
        return dates.format_time(time, format, tzinfo, self)

    def format_timedelta(self, delta, granularity='second',
                threshold=0.84999999999999998):
        """
        Return a time delta according to the rules of the given locale.

        :param delta: a timedelta object representing the time difference to
                      format, or the delta in seconds as an int value
        :param granularity: determines the smallest unit that should be
                            displayed, the value can be one of "year",
                            "month", "week", "day", "hour", "minute" or
                            "second"
        :param threshold: factor that determines at which point the
                          presentation switches to the next higher unit
        """
        return dates.format_timedelta(delta, granularity, threshold, self)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
"""

    Babel localisation support for Tornado

    :copyright: (c) 2012 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from setuptools import setup, find_packages

setup(
    name = "Tornado-Babel",
    version = "0.1",
    packages = find_packages(),

    install_requires = [
        "tornado",
        "babel",
        "speaklater",
    ],

    author = "Openlabs Technologies & Consulting (P) Limited",
    author_email = "info@openlabs.co.in",
    description = "Babel localisation support for Torando",
    license = "BSD",
    keywords = "tornado locale babel localisation",
    url = "https://github.com/openlabs/tornado-babel",
    test_suite = "tests.test_all",

    entry_points="""
    [babel.extractors]
    tornado = tornadobabel.extract:extract_tornado
    """,
)
